import unittest
import yaml

from acai_aws.apigateway.openapi.handler.importer import HandlerImporter
from acai_aws.apigateway.openapi.generator import OpenAPIGenerator


class OpenAPIGeneratorTest(unittest.TestCase):

    def setUp(self):
        self.non_existing_expected = {
            'openapi': '3.1.0',
            'info': {'version': '1.0.0', 'title': 'Acai Generator', 'license': {'name': 'MIT'}},
            'tags': [{'name': 'acai_aws-unit_test'}],
            'servers': [],
            'paths': {
                '/acai_aws/unit_test/basic': {
                    'get': {
                        'tags': ['acai_aws-unit_test'],
                        'operationId': 'GetAcaiAwsUnitTestBasicAcaiGenerated',
                        'deprecated': False,
                        'parameters': [
                            {'in': 'query', 'name': 'basic_id', 'required': True, 'schema': {'type': 'string'}},
                            {'in': 'query', 'name': 'first', 'required': False, 'schema': {'type': 'string'}},
                            {'in': 'query', 'name': 'last', 'required': False, 'schema': {'type': 'string'}}
                        ]
                    }
                }
            },
            'components': {
                'securitySchemes': {'AcaiGenerated': {'type': 'apiKey', 'in': 'header', 'name': 'CHANGE-ME'}},
                'schemas': {}
            }
        }
        self.deleted_existed_expected = {
            "openapi": "3.1.0",
            "info": {
                "version": "1.0.0",
                "title": "Acai Generator",
                "license": {
                    "name": "MIT"
                }
            },
            "tags": [
                {
                    "name": "acai_aws-unit_test"
                }
            ],
            "servers": [],
            "paths": {
                "/acai_aws/unit_test/basic": {
                    "get": {
                        "tags": [
                            "acai_aws-unit_test"
                        ],
                        "operationId": "GetAcaiAwsUnitTestBasicAcaiGenerated",
                        "deprecated": False,
                        "parameters": [
                            {
                                "in": "query",
                                "name": "basic_id",
                                "required": True,
                                "schema": {
                                    "type": "string"
                                }
                            },
                            {
                                "in": "query",
                                "name": "first",
                                "required": False,
                                "schema": {
                                    "type": "string"
                                }
                            },
                            {
                                "in": "query",
                                "name": "last",
                                "required": False,
                                "schema": {
                                    "type": "string"
                                }
                            }
                        ]
                    }
                }
            },
            "components": {
                "securitySchemes": {
                    "AcaiGenerated": {
                        "type": "apiKey",
                        "in": "header",
                        "name": "CHANGE-ME"
                    }
                },
                "schemas": {}
            }
        }
        with open('tests/mocks/apigateway/openapi/files/yml/openapi.yml', encoding='utf-8') as schema_file:
            self.existing_expected_yml = yaml.load(schema_file, Loader=yaml.FullLoader)
        with open('tests/mocks/apigateway/openapi/files/json/openapi.json', encoding='utf-8') as schema_file:
            self.existing_expected_json = yaml.load(schema_file, Loader=yaml.FullLoader)

    def test_add_path_and_method_existing_yml_file(self):
        generator = OpenAPIGenerator('tests/mocks/apigateway/openapi/files/yml')
        self.assertDictEqual(self.existing_expected_yml, generator.doc)

    def test_add_path_and_method_existing_json_file(self):
        generator = OpenAPIGenerator('tests/mocks/apigateway/openapi/files/json')
        self.assertDictEqual(self.existing_expected_json, generator.doc)

    def test_add_path_and_method_non_existing_file(self):
        importer = HandlerImporter()
        generator = OpenAPIGenerator('tests/mocks_outputs')
        module = importer.get_modules_from_file_paths(
            ['tests/mocks/apigateway/openapi/basic.py'],
            'tests/mocks/apigateway/openapi',
            'acai_aws/unit_test'
        )[0]
        generator.add_path_and_method(module)
        self.assertDictEqual(self.non_existing_expected, generator.doc)

    def test_delete_unused_paths(self):
        importer = HandlerImporter()
        generator = OpenAPIGenerator('tests/mocks_outputs')
        module = importer.get_modules_from_file_paths(
            ['tests/mocks/apigateway/openapi/basic.py'],
            'tests/mocks/apigateway/openapi',
            'acai_aws/unit_test'
        )[0]
        generator.add_path_and_method(module)
        generator.delete_unused_paths()
        self.assertDictEqual(self.deleted_existed_expected, generator.doc)



import base64
import json
import unittest

from acai_aws.msk.record import Record

from tests.mocks.msk import mock_event


class MSKRecordTest(unittest.TestCase):
    basic_record = mock_event.get_basic()['records']['mytopic-0'][0]

    def test_record_accepts_event(self):
        record = Record(self.basic_record)
        self.assertEqual(record.operation, record.UNKNOWN)
        self.assertEqual(record.topic, self.basic_record['topic'])
        self.assertEqual(record.partition, self.basic_record['partition'])
        self.assertEqual(record.offset, self.basic_record['offset'])
        self.assertEqual(record.time_stamp, self.basic_record['timestamp'])
        self.assertEqual(record.time_stamp_type, self.basic_record['timestampType'])
        self.assertEqual(record.key, base64.b64decode(self.basic_record['key']).decode('utf-8'))
        self.assertDictEqual(record.value, json.loads(base64.b64decode(self.basic_record['value']).decode('utf-8')))
        self.assertDictEqual(record.body, json.loads(base64.b64decode(self.basic_record['value']).decode('utf-8')))
        self.assertListEqual(record.headers, self.basic_record['headers'])

    def test_record_prints(self):
        try:
            record = Record(self.basic_record)
            print(record)
            self.assertTrue(True)
        except Exception as error:
            print(error)
            self.assertTrue(False)



import os
import unittest

from acai_aws.apigateway.openapi.handler.scanner import HandlerScanner


class HandlerScannerTest(unittest.TestCase):
    handler_path = 'tests/mocks/apigateway/openapi/**/*.py'

    def setUp(self):
        self.maxDiff = None
        self.scanner = HandlerScanner(self.handler_path)

    def test_file_separator(self):
        assert self.scanner.file_separator == os.sep

    def test_handlers(self):
        assert self.scanner.handlers == self.handler_path

    def test_clean_path(self):
        dirty_path = '/dirty/path/'
        result = self.scanner.clean_path(dirty_path)
        assert result == 'dirty/path'

    def test_get_handler_file_paths(self):
        paths = self.scanner.get_handler_file_paths()
        assert len(paths) == 8

    def test_get_handler_file_no_directory(self):
        scanner = HandlerScanner('tests/mocks/apigateway/openapi')
        paths = scanner.get_handler_file_paths()
        assert len(paths) == 8



from acai_aws.common.json_helper import JsonHelper


class Event:

    def __init__(self, event, context=None):
        self._event = event
        self._context = context

    @property
    def body(self):
        return JsonHelper.decode(self._event)



import base64

from acai_aws.common.json_helper import JsonHelper
from acai_aws.base.record import BaseRecord


class Record(BaseRecord):

    @property
    def message_id(self):
        return self._record.get('messageID')

    @property
    def message_type(self):
        return self._record.get('messageType')

    @property
    def data(self):
        b64_decoded = base64.b64decode(self._record.get('data')).decode('utf-8')
        return JsonHelper.decode(b64_decoded)

    @property
    def delivery_mode(self):
        return self._record.get('deliveryMode')

    @property
    def reply_to(self):
        return self._record.get('replyTo')

    @property
    def record_type(self):
        return self._record.get('type')

    @property
    def expiration(self):
        return self._record.get('expiration')

    @property
    def priority(self):
        return self._record.get('priority')

    @property
    def correlation_id(self):
        return self._record.get('correlationId')

    @property
    def redelivered(self):
        return self._record.get('redelivered')

    @property
    def destination(self):
        return self._record.get('destination')

    @property
    def properties(self):
        return self._record.get('properties')

    @property
    def time_stamp(self):
        return self._record.get('timestamp')

    @property
    def in_time(self):
        return self._record.get('brokerInTime')

    @property
    def out_time(self):
        return self._record.get('brokerOutTime')

    @property
    def operation(self):
        return self.UNKNOWN

    @property
    def body(self):
        return self.data

    def __str__(self):
        return str({
            'message_id': self.message_id,
            'message_type': self.message_type,
            'delivery_mode': self.delivery_mode,
            'reply_to': self.reply_to,
            'record_type': self.record_type,
            'expiration': self.expiration,
            'priority': self.priority,
            'correlation_id': self.correlation_id,
            'redelivered': self.redelivered,
            'destination': self.destination,
            'properties': self.properties,
            'time_stamp': self.time_stamp,
            'in_time': self.in_time,
            'out_time': self.out_time,
            'data': self.data,
            'body': self.body,
            'operation': self.operation
        })



import base64
import gzip
import json
import unittest

from acai_aws.apigateway.response import Response


class ResponseTest(unittest.TestCase):

    def setUp(self):
        self.response = Response()

    def test_defaults(self):
        self.assertEqual(False, self.response.has_errors)
        self.assertDictEqual(
            self.response.full,
            {
                'headers': {
                    'Access-Control-Allow-Origin': '*',
                    'Access-Control-Allow-Headers': '*'
                },
                'statusCode': 204,
                'isBase64Encoded': False,
                'body': '{}'
            }
        )

    def test_default_headers(self):
        self.assertDictEqual(
            self.response.headers, {
                'Access-Control-Allow-Origin': '*',
                'Access-Control-Allow-Headers': '*'
            }
        )

    def test_closed_cors_headers(self):
        self.response.cors = False
        self.assertDictEqual(self.response.headers, {})

    def test_closed_open_cors_headers(self):
        self.response.open_cors = False
        self.assertDictEqual(self.response.headers, {})
        self.assertFalse(self.response.open_cors)

    def test_header_assignment(self):
        self.response.headers = ('some-key', 'some-value')
        self.assertDictEqual(
            self.response.headers, {
                'Access-Control-Allow-Origin': '*',
                'Access-Control-Allow-Headers': '*',
                'some-key': 'some-value'
            }
        )

    def test_compress(self):
        self.response.body = {'unit-test': True}
        self.response.compress = True
        apigateway_response = self.response.full.copy()
        decoded = json.loads(gzip.decompress(base64.b64decode(apigateway_response['body'])))
        self.assertEqual(apigateway_response['isBase64Encoded'], True)
        self.assertEqual(apigateway_response['headers']['Content-Encoding'], 'gzip')
        self.assertDictEqual(decoded, {'unit-test': True})

    def test_default_code(self):
        self.response.body = {'key': 'value'}
        self.assertEqual(self.response.code, 200)

    def test_assigned_code(self):
        self.response.code = 201
        self.assertEqual(self.response.code, 201)

    def test_empty_body_default_code(self):
        self.assertEqual(self.response.code, 204)

    def test_error_body_default_code(self):
        self.response.set_error('some-error-key', 'some-error-value')
        self.assertEqual(self.response.code, 400)

    def test_is_json(self):
        self.response.is_json = False
        self.response.body = 'not json'
        self.assertEqual(self.response.body, 'not json')

    def test_set_error(self):
        self.response.set_error('some-error-key', 'some-error-value')
        self.assertEqual(
            self.response.body, '{"errors": [{"key_path": "some-error-key", "message": "some-error-value"}]}'
        )

    def test_has_error(self):
        self.response.set_error('some-error-key', 'some-error-value')
        self.assertEqual(self.response.has_errors, True)

    def test_default_content_type_with_default_is_json(self):
        self.assertEqual('application/json', self.response.content_type)

    def test_default_content_type_with_is_json_false(self):
        self.response.is_json = False
        self.assertEqual('', self.response.content_type)

    def test_default_content_type_set_will_stick(self):
        self.response.content_type = 'application/xml'
        self.assertEqual('application/xml', self.response.content_type)

    def test_default_content_type_follows_headers(self):
        self.response.headers = ('Content-Type', 'text/html')
        self.assertEqual('text/html', self.response.content_type)

    def test_raw(self):
        self.response.body = {'raw': True}
        self.assertDictEqual({'raw': True}, self.response.raw)



import simplejson as json


class JsonHelper:

    @staticmethod
    def decode(data, raise_error=False):
        try:
            return json.loads(data, use_decimal=True)
        except Exception as error:
            if raise_error:
                raise error
            return data

    @staticmethod
    def encode(data, raise_error=False):
        try:
            return json.dumps(data, use_decimal=True)
        except Exception as error:
            if raise_error:
                raise error
            return data



import json
import unittest

from acai_aws.sqs.record import Record

from tests.mocks.sqs import mock_event


class SQSRecordTest(unittest.TestCase):
    basic_record = mock_event.get_basic()['Records'][0]
    expected_attributes = {
        'ApproximateReceiveCount': '1',
        'SentTimestamp': '1545082649183',
        'SenderId': 'AIDAIENQZJOLO23YVJ4VO',
        'ApproximateFirstReceiveTimestamp': '1545082649185',
        'SomeString': 'Some String',
        'SomeBinary': 'Some Binary'
    }

    def test_record_accepts_event(self):
        record = Record(self.basic_record)
        self.assertEqual(record.operation, record.UNKNOWN)
        self.assertDictEqual(record.body, json.loads(self.basic_record['body']))
        self.assertEqual(record.message_id, self.basic_record['messageId'])
        self.assertEqual(record.receipt_handle, self.basic_record['receiptHandle'])
        self.assertEqual(record.message_attributes, self.basic_record['messageAttributes'])
        self.assertEqual(record.region, self.basic_record['awsRegion'])
        self.assertEqual(record.source_arn, self.basic_record['eventSourceARN'])
        self.assertEqual(record.source, self.basic_record['eventSource'])
        self.assertEqual(record.md5_of_body, self.basic_record['md5OfBody'])
        self.assertEqual(record.attributes, self.expected_attributes)

    def test_record_prints(self):
        try:
            record = Record(self.basic_record)
            print(record)
            self.assertTrue(True)
        except Exception as error:
            print(error)
            self.assertTrue(False)



import unittest

from acai_aws.common.records.exception import EventTimeOutException

from tests.mocks.generic import mock_event
from tests.mocks.generic.mock_class import MockDataClass
from tests.mocks.generic.mock_functions import mock_generic, mock_timeout, before_call, after_call, call_list, mock_generic_dc


class GenericRequirementsTest(unittest.TestCase):
    basic_event = mock_event.get_basic()
    expected = {'generic': {'key': 'value'}}

    def test_generic_decorator_with_basic_requirements(self):
        result = mock_generic(self.basic_event, None)
        self.assertDictEqual(result, self.expected)

    def test_generic_decorator_with_full_requirements(self):
        result = mock_generic(self.basic_event, None)
        self.assertDictEqual(result, self.expected)
        self.assertTrue(before_call.has_been_called)
        self.assertTrue(after_call.has_been_called)
        self.assertEqual('before', call_list[0])
        self.assertEqual('after', call_list[1])

    def test_generic_decorator_with_data_class(self):
        result = mock_generic_dc(self.basic_event, None)
        self.assertTrue(isinstance(result, MockDataClass))

    def test_decorator_with_timeout(self):
        try:
            mock_timeout(self.basic_event, None)
            self.assertTrue(False)
        except EventTimeOutException as error:
            self.assertTrue(isinstance(error, EventTimeOutException))



import base64

from acai_aws.common.json_helper import JsonHelper
from acai_aws.base.record import BaseRecord


class Record(BaseRecord):

    @property
    def topic(self):
        return self._record.get('topic')

    @property
    def partition(self):
        return self._record.get('partition')

    @property
    def offset(self):
        return self._record.get('offset')

    @property
    def time_stamp(self):
        return self._record.get('timestamp')

    @property
    def time_stamp_type(self):
        return self._record.get('timestampType')

    @property
    def key(self):
        return base64.b64decode(self._record.get('key')).decode('utf-8')

    @property
    def value(self):
        b64_decoded = base64.b64decode(self._record.get('value')).decode('utf-8')
        return JsonHelper.decode(b64_decoded)

    @property
    def headers(self):
        return self._record.get('headers')

    @property
    def operation(self):
        return self.UNKNOWN

    @property
    def body(self):
        return self.value

    def __str__(self):
        return str({
            'topic': self.topic,
            'partition': self.partition,
            'offset': self.offset,
            'time_stamp': self.time_stamp,
            'time_stamp_type': self.time_stamp_type,
            'key': self.key,
            'value': self.value,
            'headers': self.headers,
            'operation': self.operation,
            'body': self.body
        })



import os
from setuptools import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='acai_aws',
    version=os.getenv('CIRCLE_TAG', '0.0.0'),
    url='https://github.com/syngenta/acai-python',
    author='Paul Cruse III',
    author_email='paulcruse3@gmail.com',
    description='DRY, configurable, declarative framework for building AWS Lambda APIs and event processors',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    include_package_data=True,
    python_requires='>=3.8',
    install_requires=[
        'boto3',
        'dynamodb_json',
        'jsonpickle',
        'jsonref',
        'jsonschema',
        'icecream',
        'pydantic',
        'pyyaml',
        'simplejson',
        'xmltodict'
    ],
    keywords=[
        'aws', 'lambda', 'serverless', 'apigateway', 'router', 'openapi', 'pydantic',
        'dynamodb', 'sqs', 'sns', 's3', 'kinesis', 'firehose', 'msk', 'documentdb',
        'event-driven', 'validation', 'jsonschema', 'python-framework'
    ],
    project_urls={
        'Homepage': 'https://github.com/syngenta/acai-python',
        'Documentation': 'https://syngenta.github.io/acai-python-docs/',
        'Source': 'https://github.com/syngenta/acai-python',
        'Issue Tracker': 'https://github.com/syngenta/acai-python/issues',
        'CI/CD': 'https://circleci.com/gh/syngenta/acai-python'
    },
    license='Apache 2.0',
    platforms=['any'],
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: Apache Software License',
        'Environment :: Web Environment',
        'Environment :: Console',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: System :: Distributed Computing',
        'Topic :: System :: Monitoring',
        'Natural Language :: English'
    ]
)



import unittest

from acai_aws.apigateway.openapi.handler.importer import HandlerImporter


class HandlerImporterTest(unittest.TestCase):

    def test_get_modules_from_file_paths(self):
        importer = HandlerImporter()
        file_paths = ['tests/mocks/apigateway/openapi/basic.py']
        handlers_base = 'tests/mocks/apigateway/openapi'
        base_path = 'acai_aws/example'
        modules = importer.get_modules_from_file_paths(file_paths, handlers_base, base_path)
        assert len(modules) == 2


import os
import unittest
from unittest.mock import patch

from acai_aws.apigateway.__main__ import generate_openapi


def make_directory(directory):
    try:
        os.mkdir(directory)
    except:
        pass


class MainTest(unittest.TestCase):

    def setUp(self):
        make_directory('tests/outputs')
        make_directory('tests/outputs/main')

    @patch('sys.argv', [
        '__main__',
        'generate-openapi',
        '--base=acai_aws/example',
        '--handlers=tests/mocks/apigateway/openapi/**/*.py',
        '--output=tests/outputs/main',
        '--format=json,yml',
        '--delete'
    ])
    def test_main(self):
        generate_openapi()
        self.assertTrue(os.path.exists('tests/outputs/main/openapi.yml'))
        self.assertTrue(os.path.exists('tests/outputs/main/openapi.json'))
